"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFromJson = exports.createFromIdl = exports.createFromRoot = void 0;
const nodes_1 = require("./nodes");
const shared_1 = require("./shared");
const visitors_1 = require("./visitors");
function createFromRoot(root, useDefaultVisitor = true) {
    let currentRoot = root;
    validateKinobiVersion(currentRoot.version);
    if (useDefaultVisitor) {
        currentRoot = (0, visitors_1.visit)(currentRoot, (0, visitors_1.defaultVisitor)());
    }
    return {
        getRoot() {
            return currentRoot;
        },
        getJson() {
            return JSON.stringify(currentRoot);
        },
        accept(visitor) {
            return (0, visitors_1.visit)(currentRoot, visitor);
        },
        update(visitor) {
            const newRoot = (0, visitors_1.visit)(currentRoot, visitor);
            (0, nodes_1.assertIsNode)(newRoot, 'rootNode');
            currentRoot = newRoot;
        },
        clone() {
            return createFromRoot({ ...currentRoot }, false);
        },
    };
}
exports.createFromRoot = createFromRoot;
function createFromIdl(program, additionalPrograms, useDefaultVisitor = true) {
    return createFromRoot((0, nodes_1.rootNodeFromIdls)(program, additionalPrograms), useDefaultVisitor);
}
exports.createFromIdl = createFromIdl;
function createFromJson(json, useDefaultVisitor = false) {
    return createFromRoot(JSON.parse(json), useDefaultVisitor);
}
exports.createFromJson = createFromJson;
function validateKinobiVersion(rootVersion) {
    // TODO: Replace with __VERSION__ variable when available.
    const kinobiVersion = '0.19.0';
    if (rootVersion === kinobiVersion)
        return;
    const [rootMajor, rootMinor] = rootVersion.split('.').map(Number);
    const [KinobiMajor, KinobiMinor] = kinobiVersion.split('.').map(Number);
    const isZeroMajor = rootMajor === 0 && KinobiMajor === 0;
    if (isZeroMajor && rootMinor === KinobiMinor)
        return;
    if (rootMajor === KinobiMajor)
        return;
    throw new shared_1.KinobiError(`The provided IDL version [${rootVersion}] is not compatible with the installed Kinobi version [${kinobiVersion}]`);
}
//# sourceMappingURL=Kinobi.js.map