"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.accountNodeFromIdl = exports.accountNode = void 0;
const shared_1 = require("../shared");
const Node_1 = require("./Node");
const linkNodes_1 = require("./linkNodes");
const typeNodes_1 = require("./typeNodes");
const TypeNode_1 = require("./typeNodes/TypeNode");
function accountNode(input) {
    var _a, _b, _c;
    if (!input.name) {
        throw new shared_1.InvalidKinobiTreeError('AccountNode must have a name.');
    }
    return {
        kind: 'accountNode',
        // Children.
        data: ((_a = input.data) !== null && _a !== void 0 ? _a : (0, typeNodes_1.structTypeNode)([])),
        pda: input.pda,
        discriminators: input.discriminators,
        // Data.
        name: (0, shared_1.mainCase)(input.name),
        idlName: (_b = input.idlName) !== null && _b !== void 0 ? _b : input.name,
        docs: (_c = input.docs) !== null && _c !== void 0 ? _c : [],
        size: input.size,
    };
}
exports.accountNode = accountNode;
function accountNodeFromIdl(idl) {
    var _a, _b, _c, _d;
    const idlName = (_a = idl.name) !== null && _a !== void 0 ? _a : '';
    const name = (0, shared_1.mainCase)(idlName);
    const idlStruct = (_b = idl.type) !== null && _b !== void 0 ? _b : { kind: 'struct', fields: [] };
    const data = (0, TypeNode_1.createTypeNodeFromIdl)(idlStruct);
    (0, Node_1.assertIsNode)(data, 'structTypeNode');
    const hasSeeds = ((_c = idl.seeds) !== null && _c !== void 0 ? _c : []).length > 0;
    return accountNode({
        name,
        data,
        pda: hasSeeds ? (0, linkNodes_1.pdaLinkNode)(name) : undefined,
        idlName,
        docs: (_d = idl.docs) !== null && _d !== void 0 ? _d : [],
        size: idl.size,
    });
}
exports.accountNodeFromIdl = accountNodeFromIdl;
//# sourceMappingURL=AccountNode.js.map