"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertIsNestedTypeNode = exports.isNestedTypeNode = exports.transformNestedTypeNode = exports.resolveNestedTypeNode = void 0;
const Node_1 = require("../Node");
const TypeNode_1 = require("./TypeNode");
function resolveNestedTypeNode(typeNode) {
    switch (typeNode.kind) {
        case 'fixedSizeTypeNode':
        case 'hiddenPrefixTypeNode':
        case 'hiddenSuffixTypeNode':
        case 'postOffsetTypeNode':
        case 'preOffsetTypeNode':
        case 'sentinelTypeNode':
        case 'sizePrefixTypeNode':
            return resolveNestedTypeNode(typeNode.type);
        default:
            return typeNode;
    }
}
exports.resolveNestedTypeNode = resolveNestedTypeNode;
function transformNestedTypeNode(typeNode, map) {
    switch (typeNode.kind) {
        case 'fixedSizeTypeNode':
        case 'hiddenPrefixTypeNode':
        case 'hiddenSuffixTypeNode':
        case 'postOffsetTypeNode':
        case 'preOffsetTypeNode':
        case 'sentinelTypeNode':
        case 'sizePrefixTypeNode':
            return {
                ...typeNode,
                type: transformNestedTypeNode(typeNode.type, map),
            };
        default:
            return map(typeNode);
    }
}
exports.transformNestedTypeNode = transformNestedTypeNode;
function isNestedTypeNode(node, kind) {
    if (!(0, Node_1.isNode)(node, TypeNode_1.TYPE_NODES))
        return false;
    const kinds = Array.isArray(kind) ? kind : [kind];
    const resolved = resolveNestedTypeNode(node);
    return !!node && kinds.includes(resolved.kind);
}
exports.isNestedTypeNode = isNestedTypeNode;
function assertIsNestedTypeNode(node, kind) {
    var _a;
    const kinds = Array.isArray(kind) ? kind : [kind];
    if (!isNestedTypeNode(node, kinds)) {
        throw new Error(`Expected nested type of ${kinds.join(' | ')}, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertIsNestedTypeNode = assertIsNestedTypeNode;
//# sourceMappingURL=NestedTypeNode.js.map