"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionParseFunctionFragment = void 0;
const common_1 = require("./common");
function getInstructionParseFunctionFragment(scope) {
    const { instructionNode, programNode, dataArgsManifest, nameApi, customInstructionData, } = scope;
    const customData = customInstructionData.get(instructionNode.name);
    const hasAccounts = instructionNode.accounts.length > 0;
    const hasOptionalAccounts = instructionNode.accounts.some((account) => account.isOptional);
    const minimumNumberOfAccounts = instructionNode.optionalAccountStrategy === 'omitted'
        ? instructionNode.accounts.filter((account) => !account.isOptional).length
        : instructionNode.accounts.length;
    const hasData = !!customData || instructionNode.arguments.length > 0;
    if (!hasAccounts && !hasData) {
        return (0, common_1.fragment)('');
    }
    const instructionDataName = nameApi.instructionDataType(instructionNode.name);
    const programAddressConstant = nameApi.programAddressConstant(programNode.name);
    const dataTypeFragment = (0, common_1.fragment)(customData
        ? dataArgsManifest.strictType.render
        : nameApi.dataType(instructionDataName));
    const decoderFunction = customData
        ? dataArgsManifest.decoder.render
        : `${nameApi.decoderFunction(instructionDataName)}()`;
    if (customData) {
        dataTypeFragment.mergeImportsWith(dataArgsManifest.strictType, dataArgsManifest.decoder);
    }
    return (0, common_1.fragmentFromTemplate)('instructionParseFunction.njk', {
        instruction: instructionNode,
        programAddressConstant,
        instructionParsedType: nameApi.instructionParsedType(instructionNode.name),
        instructionParseFunction: nameApi.instructionParseFunction(instructionNode.name),
        dataTypeFragment,
        decoderFunction,
        hasAccounts,
        hasOptionalAccounts,
        minimumNumberOfAccounts,
        hasData,
    })
        .mergeImportsWith(dataTypeFragment)
        .addImports('generatedPrograms', [programAddressConstant])
        .addImports('solanaInstructions', ['IInstruction'])
        .addImports('solanaInstructions', hasAccounts ? ['IInstructionWithAccounts', 'IAccountMeta'] : [])
        .addImports('solanaInstructions', hasData ? ['IInstructionWithData'] : []);
}
exports.getInstructionParseFunctionFragment = getInstructionParseFunctionFragment;
//# sourceMappingURL=instructionParseFunction.js.map