"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionTypeFragment = void 0;
const shared_1 = require("../../../shared");
const common_1 = require("./common");
const instructionAccountMeta_1 = require("./instructionAccountMeta");
const instructionAccountTypeParam_1 = require("./instructionAccountTypeParam");
function getInstructionTypeFragment(scope) {
    const { instructionNode, programNode, nameApi, customInstructionData } = scope;
    const hasAccounts = instructionNode.accounts.length > 0;
    const customData = customInstructionData.get(instructionNode.name);
    const hasData = !!customData || instructionNode.arguments.length > 0;
    const instructionDataName = nameApi.instructionDataType(instructionNode.name);
    const programAddressConstant = nameApi.programAddressConstant(programNode.name);
    const dataType = customData
        ? (0, shared_1.pascalCase)(customData.importAs)
        : (0, shared_1.pascalCase)(instructionDataName);
    const accountTypeParamsFragment = (0, common_1.mergeFragments)(instructionNode.accounts.map((account) => (0, instructionAccountTypeParam_1.getInstructionAccountTypeParamFragment)({
        ...scope,
        instructionAccountNode: account,
        allowAccountMeta: true,
    })), (renders) => renders.join(', '));
    const usesLegacyOptionalAccounts = instructionNode.optionalAccountStrategy === 'omitted';
    const accountMetasFragment = (0, common_1.mergeFragments)(instructionNode.accounts.map((account) => (0, instructionAccountMeta_1.getInstructionAccountMetaFragment)(account).mapRender((r) => {
        const typeParam = `TAccount${(0, shared_1.pascalCase)(account.name)}`;
        const isLegacyOptional = account.isOptional && usesLegacyOptionalAccounts;
        const type = `${typeParam} extends string ? ${r} : ${typeParam}`;
        if (!isLegacyOptional)
            return type;
        return `...(${typeParam} extends undefined ? [] : [${type}])`;
    })), (renders) => renders.join(', '));
    const fragment = (0, common_1.fragmentFromTemplate)('instructionType.njk', {
        instruction: instructionNode,
        instructionType: nameApi.instructionType(instructionNode.name),
        programAddressConstant,
        hasData,
        hasAccounts,
        dataType,
        accountTypeParams: accountTypeParamsFragment.render,
        accountMetas: accountMetasFragment.render,
    })
        .mergeImportsWith(accountTypeParamsFragment, accountMetasFragment)
        .addImports('generatedPrograms', [programAddressConstant])
        .addImports('solanaInstructions', [
        'IAccountMeta',
        'IInstruction',
        'IInstructionWithAccounts',
        ...(hasData ? ['IInstructionWithData'] : []),
    ]);
    // TODO: if link, add import for data type. Unless we don't need to inject the data type in IInstructionWithData.
    return fragment;
}
exports.getInstructionTypeFragment = getInstructionTypeFragment;
//# sourceMappingURL=instructionType.js.map