"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProgramInstructionsFragment = void 0;
const nodes_1 = require("../../../nodes");
const common_1 = require("./common");
const discriminatorCondition_1 = require("./discriminatorCondition");
function getProgramInstructionsFragment(scope) {
    if (scope.programNode.instructions.length === 0)
        return (0, common_1.fragment)('');
    const allInstructions = (0, nodes_1.getAllInstructionsWithSubs)(scope.programNode, {
        leavesOnly: !scope.renderParentInstructions,
        subInstructionsFirst: true,
    });
    const scopeWithInstructions = { ...scope, allInstructions };
    return (0, common_1.mergeFragments)([
        getProgramInstructionsEnumFragment(scopeWithInstructions),
        getProgramInstructionsIdentifierFunctionFragment(scopeWithInstructions),
        getProgramInstructionsParsedUnionTypeFragment(scopeWithInstructions),
    ], (r) => `${r.join('\n\n')}\n`);
}
exports.getProgramInstructionsFragment = getProgramInstructionsFragment;
function getProgramInstructionsEnumFragment(scope) {
    const { programNode, allInstructions, nameApi } = scope;
    const programInstructionsEnum = nameApi.programInstructionsEnum(programNode.name);
    const programInstructionsEnumVariants = allInstructions.map((instruction) => nameApi.programInstructionsEnumVariant(instruction.name));
    return (0, common_1.fragment)(`export enum ${programInstructionsEnum} { ` +
        `${programInstructionsEnumVariants.join(', ')}` +
        ` }`);
}
function getProgramInstructionsIdentifierFunctionFragment(scope) {
    const { programNode, nameApi, allInstructions } = scope;
    const instructionsWithDiscriminators = allInstructions.filter((instruction) => { var _a; return ((_a = instruction.discriminators) !== null && _a !== void 0 ? _a : []).length > 0; });
    const hasInstructionDiscriminators = instructionsWithDiscriminators.length > 0;
    if (!hasInstructionDiscriminators)
        return (0, common_1.fragment)('');
    const programInstructionsEnum = nameApi.programInstructionsEnum(programNode.name);
    const programInstructionsIdentifierFunction = nameApi.programInstructionsIdentifierFunction(programNode.name);
    const discriminatorsFragment = (0, common_1.mergeFragments)(instructionsWithDiscriminators.map((instruction) => {
        var _a;
        const variant = nameApi.programInstructionsEnumVariant(instruction.name);
        return (0, discriminatorCondition_1.getDiscriminatorConditionFragment)({
            ...scope,
            discriminators: (_a = instruction.discriminators) !== null && _a !== void 0 ? _a : [],
            struct: (0, nodes_1.structTypeNodeFromInstructionArgumentNodes)(instruction.arguments),
            dataName: 'data',
            ifTrue: `return ${programInstructionsEnum}.${variant};`,
        });
    }), (r) => r.join('\n'));
    return discriminatorsFragment.mapRender((discriminators) => `export function ${programInstructionsIdentifierFunction}(` +
        `instruction: { data: Uint8Array } | Uint8Array` +
        `): ${programInstructionsEnum} {\n` +
        `const data = instruction instanceof Uint8Array ? instruction : instruction.data;\n` +
        `${discriminators}\n` +
        `throw new Error("The provided instruction could not be identified as a ${programNode.name} instruction.")\n` +
        `}`);
}
function getProgramInstructionsParsedUnionTypeFragment(scope) {
    const { programNode, allInstructions, nameApi } = scope;
    const programAddress = programNode.publicKey;
    const programInstructionsType = nameApi.programInstructionsParsedUnionType(programNode.name);
    const programInstructionsEnum = nameApi.programInstructionsEnum(programNode.name);
    const typeVariants = allInstructions.map((instruction) => {
        const instructionEnumVariant = nameApi.programInstructionsEnumVariant(instruction.name);
        const parsedInstructionType = nameApi.instructionParsedType(instruction.name);
        return (0, common_1.fragment)(`| { instructionType: ${programInstructionsEnum}.${instructionEnumVariant} } & ${parsedInstructionType}<TProgram>`).addImports('generatedInstructions', parsedInstructionType);
    });
    return (0, common_1.mergeFragments)([
        (0, common_1.fragment)(`export type ${programInstructionsType}<TProgram extends string = '${programAddress}'> =`),
        ...typeVariants,
    ], (r) => r.join('\n'));
}
//# sourceMappingURL=programInstructions.js.map