"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEFAULT_NAME_TRANSFORMERS = exports.getNameApi = void 0;
const shared_1 = require("../../shared");
function getNameApi(transformers) {
    const helpers = {
        capitalize: shared_1.capitalize,
        titleCase: shared_1.titleCase,
        pascalCase: shared_1.pascalCase,
        camelCase: shared_1.camelCase,
        kebabCase: shared_1.kebabCase,
        snakeCase: shared_1.snakeCase,
    };
    return Object.fromEntries(Object.entries(transformers).map(([key, transformer]) => [
        key,
        (name) => transformer(name, helpers),
    ]));
}
exports.getNameApi = getNameApi;
exports.DEFAULT_NAME_TRANSFORMERS = {
    dataType: (name) => `${(0, shared_1.pascalCase)(name)}`,
    dataArgsType: (name) => `${(0, shared_1.pascalCase)(name)}Args`,
    encoderFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Encoder`,
    decoderFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Decoder`,
    codecFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Codec`,
    pdaSeedsType: (name) => `${(0, shared_1.pascalCase)(name)}Seeds`,
    pdaFindFunction: (name) => `find${(0, shared_1.pascalCase)(name)}Pda`,
    accountType: (name) => `${(0, shared_1.pascalCase)(name)}`,
    accountMaybeType: (name) => `Maybe${(0, shared_1.pascalCase)(name)}`,
    accountDataType: (name) => `${(0, shared_1.pascalCase)(name)}AccountData`,
    accountDecodeFunction: (name) => `decode${(0, shared_1.pascalCase)(name)}`,
    accountFetchFunction: (name) => `fetch${(0, shared_1.pascalCase)(name)}`,
    accountFetchAllFunction: (name) => `fetchAll${(0, shared_1.pascalCase)(name)}`,
    accountFetchMaybeFunction: (name) => `fetchMaybe${(0, shared_1.pascalCase)(name)}`,
    accountFetchAllMaybeFunction: (name) => `fetchAllMaybe${(0, shared_1.pascalCase)(name)}`,
    accountFetchFromSeedsFunction: (name) => `fetch${(0, shared_1.pascalCase)(name)}FromSeeds`,
    accountFetchMaybeFromSeedsFunction: (name) => `fetchMaybe${(0, shared_1.pascalCase)(name)}FromSeeds`,
    accountGetSizeFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Size`,
    enumVariant: (name) => `${(0, shared_1.pascalCase)(name)}`,
    discriminatedUnionDiscriminator: () => '__kind',
    discriminatedUnionVariant: (name) => `${(0, shared_1.pascalCase)(name)}`,
    discriminatedUnionFunction: (name) => `${(0, shared_1.camelCase)(name)}`,
    isDiscriminatedUnionFunction: (name) => `is${(0, shared_1.pascalCase)(name)}`,
    instructionAsyncInputType: (name) => `${(0, shared_1.pascalCase)(name)}AsyncInput`,
    instructionSyncInputType: (name) => `${(0, shared_1.pascalCase)(name)}Input`,
    instructionType: (name) => `${(0, shared_1.pascalCase)(name)}Instruction`,
    instructionDataType: (name) => `${(0, shared_1.pascalCase)(name)}InstructionData`,
    instructionExtraType: (name) => `${(0, shared_1.pascalCase)(name)}InstructionExtra`,
    instructionAsyncFunction: (name) => `get${(0, shared_1.pascalCase)(name)}InstructionAsync`,
    instructionSyncFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Instruction`,
    instructionParsedType: (name) => `Parsed${(0, shared_1.pascalCase)(name)}Instruction`,
    instructionParseFunction: (name) => `parse${(0, shared_1.pascalCase)(name)}Instruction`,
    programType: (name) => `${(0, shared_1.pascalCase)(name)}Program`,
    programAddressConstant: (name) => `${(0, shared_1.snakeCase)(name).toUpperCase()}_PROGRAM_ADDRESS`,
    programCreateFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Program`,
    programAccountsEnum: (name) => `${(0, shared_1.pascalCase)(name)}Account`,
    programAccountsEnumVariant: (name) => `${(0, shared_1.pascalCase)(name)}`,
    programAccountsIdentifierFunction: (name) => `identify${(0, shared_1.pascalCase)(name)}Account`,
    programInstructionsEnum: (name) => `${(0, shared_1.pascalCase)(name)}Instruction`,
    programInstructionsEnumVariant: (name) => `${(0, shared_1.pascalCase)(name)}`,
    programInstructionsIdentifierFunction: (name) => `identify${(0, shared_1.pascalCase)(name)}Instruction`,
    programInstructionsParsedUnionType: (name) => `Parsed${(0, shared_1.pascalCase)(name)}Instruction`,
    programErrorClass: (name) => `${(0, shared_1.pascalCase)(name)}ProgramError`,
    programErrorCodeEnum: (name) => `${(0, shared_1.pascalCase)(name)}ProgramErrorCode`,
    programErrorCodeMap: (name) => `${(0, shared_1.camelCase)(name)}ProgramErrorCodeMap`,
    programGetErrorFromCodeFunction: (name) => `get${(0, shared_1.pascalCase)(name)}ProgramErrorFromCode`,
    resolverFunction: (name) => `${(0, shared_1.camelCase)(name)}`,
};
//# sourceMappingURL=nameTransformers.js.map