"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LinkableDictionary = void 0;
const nodes_1 = require("../nodes");
class LinkableDictionary {
    constructor() {
        this.programs = new Map();
        this.pdas = new Map();
        this.accounts = new Map();
        this.definedTypes = new Map();
    }
    record(node) {
        if ((0, nodes_1.isNode)(node, 'programNode')) {
            this.programs.set(node.name, node);
        }
        if ((0, nodes_1.isNode)(node, 'pdaNode')) {
            this.pdas.set(node.name, node);
        }
        if ((0, nodes_1.isNode)(node, 'accountNode')) {
            this.accounts.set(node.name, node);
        }
        if ((0, nodes_1.isNode)(node, 'definedTypeNode')) {
            this.definedTypes.set(node.name, node);
        }
        return this;
    }
    recordAll(nodes) {
        nodes.forEach((node) => this.record(node));
        return this;
    }
    get(linkNode) {
        if (linkNode.importFrom) {
            return undefined;
        }
        if ((0, nodes_1.isNode)(linkNode, 'programLinkNode')) {
            return this.programs.get(linkNode.name);
        }
        if ((0, nodes_1.isNode)(linkNode, 'pdaLinkNode')) {
            return this.pdas.get(linkNode.name);
        }
        if ((0, nodes_1.isNode)(linkNode, 'accountLinkNode')) {
            return this.accounts.get(linkNode.name);
        }
        if ((0, nodes_1.isNode)(linkNode, 'definedTypeLinkNode')) {
            return this.definedTypes.get(linkNode.name);
        }
        return undefined;
    }
    has(linkNode) {
        if (linkNode.importFrom) {
            return false;
        }
        if ((0, nodes_1.isNode)(linkNode, 'programLinkNode')) {
            return this.programs.has(linkNode.name);
        }
        if ((0, nodes_1.isNode)(linkNode, 'pdaLinkNode')) {
            return this.pdas.has(linkNode.name);
        }
        if ((0, nodes_1.isNode)(linkNode, 'accountLinkNode')) {
            return this.accounts.has(linkNode.name);
        }
        if ((0, nodes_1.isNode)(linkNode, 'definedTypeLinkNode')) {
            return this.definedTypes.has(linkNode.name);
        }
        return false;
    }
}
exports.LinkableDictionary = LinkableDictionary;
//# sourceMappingURL=LinkableDictionary.js.map