"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getConjunctiveNodeSelectorFunction = exports.getNodeSelectorFunction = void 0;
const utils_1 = require("./utils");
const getNodeSelectorFunction = (selector) => {
    if (typeof selector === 'function')
        return selector;
    const checkNode = (node, nodeSelector) => {
        if (nodeSelector === '*')
            return true;
        const matches = nodeSelector.match(/^(?:\[([^\]]+)\])?(.*)?$/);
        if (!matches)
            return false;
        const [, kinds, name] = matches;
        // Check kinds.
        const kindArray = kinds ? kinds.split('|').map(utils_1.mainCase) : [];
        if (kindArray.length > 0 &&
            !kindArray.includes(node.kind)) {
            return false;
        }
        // Check names.
        if (name && (!('name' in node) || (0, utils_1.mainCase)(name) !== node.name)) {
            return false;
        }
        return true;
    };
    const checkStack = (nodeStack, nodeSelectors) => {
        if (nodeSelectors.length === 0)
            return true;
        if (nodeStack.length === 0)
            return false;
        const lastNode = nodeStack.pop();
        const lastNodeSelector = nodeSelectors.pop();
        return checkNode(lastNode, lastNodeSelector)
            ? checkStack(nodeStack, nodeSelectors)
            : checkStack(nodeStack, [...nodeSelectors, lastNodeSelector]);
    };
    const nodeSelectors = selector.split('.');
    const lastNodeSelector = nodeSelectors.pop();
    return (node, stack) => checkNode(node, lastNodeSelector) &&
        checkStack(stack.all(), [...nodeSelectors]);
};
exports.getNodeSelectorFunction = getNodeSelectorFunction;
const getConjunctiveNodeSelectorFunction = (selector) => {
    const selectors = Array.isArray(selector) ? selector : [selector];
    const selectorFunctions = selectors.map(exports.getNodeSelectorFunction);
    return (node, stack) => selectorFunctions.every((fn) => fn(node, stack));
};
exports.getConjunctiveNodeSelectorFunction = getConjunctiveNodeSelectorFunction;
//# sourceMappingURL=NodeSelector.js.map