"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addPdasVisitor = void 0;
const shared_1 = require("../shared");
const nodes_1 = require("../nodes");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
function addPdasVisitor(pdas) {
    return (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)(Object.entries(pdas).map(([uncasedProgramName, newPdas]) => {
        const programName = (0, shared_1.mainCase)(uncasedProgramName);
        return {
            select: `[programNode]${programName}`,
            transform: (node) => {
                (0, nodes_1.assertIsNode)(node, 'programNode');
                const existingPdaNames = new Set(node.pdas.map((pda) => pda.name));
                const newPdaNames = new Set(newPdas.map((pda) => pda.name));
                const overlappingPdaNames = new Set([...existingPdaNames].filter((name) => newPdaNames.has(name)));
                if (overlappingPdaNames.size > 0) {
                    throw new shared_1.KinobiError(`Cannot add PDAs to program "${programName}" because the following PDA names ` +
                        `already exist: ${[...overlappingPdaNames].join(', ')}.`);
                }
                return (0, nodes_1.programNode)({
                    ...node,
                    pdas: [
                        ...node.pdas,
                        ...newPdas.map((pda) => (0, nodes_1.pdaNode)(pda.name, pda.seeds)),
                    ],
                });
            },
        };
    }));
}
exports.addPdasVisitor = addPdasVisitor;
//# sourceMappingURL=addPdasVisitor.js.map