"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deduplicateIdenticalDefinedTypesVisitor = void 0;
const nodes_1 = require("../nodes");
const deleteNodesVisitor_1 = require("./deleteNodesVisitor");
const getUniqueHashStringVisitor_1 = require("./getUniqueHashStringVisitor");
const singleNodeVisitor_1 = require("./singleNodeVisitor");
const visitor_1 = require("./visitor");
function deduplicateIdenticalDefinedTypesVisitor() {
    return (0, singleNodeVisitor_1.rootNodeVisitor)((root) => {
        const typeMap = new Map();
        // Fill the type map with all defined types.
        const allPrograms = (0, nodes_1.getAllPrograms)(root);
        allPrograms.forEach((program) => {
            program.definedTypes.forEach((type) => {
                var _a;
                const typeWithProgram = { program, type };
                const list = (_a = typeMap.get(type.name)) !== null && _a !== void 0 ? _a : [];
                typeMap.set(type.name, [...list, typeWithProgram]);
            });
        });
        // Remove all types that are not duplicated.
        typeMap.forEach((list, name) => {
            if (list.length <= 1) {
                typeMap.delete(name);
            }
        });
        // Remove duplicates whose types are not equal.
        const hashVisitor = (0, getUniqueHashStringVisitor_1.getUniqueHashStringVisitor)({ removeDocs: true });
        typeMap.forEach((list, name) => {
            const types = list.map((item) => (0, visitor_1.visit)(item.type, hashVisitor));
            const typesAreEqual = types.every((type, i, arr) => type === arr[0]);
            if (!typesAreEqual) {
                typeMap.delete(name);
            }
        });
        // Get the selectors for all defined types that needs deleting.
        // Thus, we must select all but the first duplicate of each list.
        const deleteSelectors = Array.from(typeMap.values())
            // Order lists by program index, get their tails and flatten.
            .flatMap((list) => {
            const sortedList = list.sort((a, b) => allPrograms.indexOf(a.program) - allPrograms.indexOf(b.program));
            const [, ...sortedListTail] = sortedList;
            return sortedListTail;
        })
            // Get selectors from the defined types and their programs.
            .map(({ program, type }) => `[programNode]${program.name}.[definedTypeNode]${type.name}`);
        // Delete the identified nodes if any.
        if (deleteSelectors.length > 0) {
            const newRoot = (0, visitor_1.visit)(root, (0, deleteNodesVisitor_1.deleteNodesVisitor)(deleteSelectors));
            (0, nodes_1.assertIsNode)(newRoot, 'rootNode');
            return newRoot;
        }
        return root;
    });
}
exports.deduplicateIdenticalDefinedTypesVisitor = deduplicateIdenticalDefinedTypesVisitor;
//# sourceMappingURL=deduplicateIdenticalDefinedTypesVisitor.js.map