"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.flattenStruct = exports.flattenStructVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const logs_1 = require("../shared/logs");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
function flattenStructVisitor(map) {
    return (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)(Object.entries(map).map(([stack, options]) => ({
        select: `${stack}.[structTypeNode]`,
        transform: (node) => (0, exports.flattenStruct)(node, options),
    })));
}
exports.flattenStructVisitor = flattenStructVisitor;
const flattenStruct = (node, options = '*') => {
    (0, nodes_1.assertIsNode)(node, 'structTypeNode');
    const camelCaseOptions = options === '*' ? options : options.map(shared_1.camelCase);
    const shouldInline = (field) => options === '*' || camelCaseOptions.includes((0, shared_1.camelCase)(field.name));
    const inlinedFields = node.fields.flatMap((field) => {
        if ((0, nodes_1.isNode)(field.type, 'structTypeNode') && shouldInline(field)) {
            return field.type.fields;
        }
        return [field];
    });
    const inlinedFieldsNames = inlinedFields.map((arg) => arg.name);
    const duplicates = inlinedFieldsNames.filter((e, i, a) => a.indexOf(e) !== i);
    const uniqueDuplicates = [...new Set(duplicates)];
    const hasConflictingNames = uniqueDuplicates.length > 0;
    if (hasConflictingNames) {
        (0, logs_1.logWarn)(`Cound not flatten the attributes of a struct ` +
            `since this would cause the following attributes ` +
            `to conflict [${uniqueDuplicates.join(', ')}].` +
            'You may want to rename the conflicting attributes.');
    }
    return hasConflictingNames ? node : (0, nodes_1.structTypeNode)(inlinedFields);
};
exports.flattenStruct = flattenStruct;
//# sourceMappingURL=flattenStructVisitor.js.map