"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setAnchorDiscriminatorsVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const extendVisitor_1 = require("./extendVisitor");
const nonNullableIdentityVisitor_1 = require("./nonNullableIdentityVisitor");
function setAnchorDiscriminatorsVisitor() {
    let program = null;
    return (0, shared_1.pipe)((0, nonNullableIdentityVisitor_1.nonNullableIdentityVisitor)([
        'rootNode',
        'programNode',
        'accountNode',
        'instructionNode',
    ]), (v) => (0, extendVisitor_1.extendVisitor)(v, {
        visitProgram(node, { next }) {
            program = node;
            const newNode = next(node);
            program = null;
            return newNode;
        },
        visitAccount(node) {
            var _a;
            const shouldAddDiscriminator = (program === null || program === void 0 ? void 0 : program.origin) === 'anchor';
            if (!shouldAddDiscriminator)
                return node;
            const discriminatorArgument = (0, nodes_1.structFieldTypeNode)({
                name: 'discriminator',
                type: (0, nodes_1.arrayTypeNode)((0, nodes_1.numberTypeNode)('u8'), (0, nodes_1.fixedCountNode)(8)),
                defaultValue: (0, shared_1.getAnchorAccountDiscriminator)(node.idlName),
                defaultValueStrategy: 'omitted',
            });
            return (0, nodes_1.accountNode)({
                ...node,
                discriminators: [
                    (0, nodes_1.fieldDiscriminatorNode)('discriminator'),
                    ...((_a = node.discriminators) !== null && _a !== void 0 ? _a : []),
                ],
                data: (0, nodes_1.transformNestedTypeNode)(node.data, (struct) => (0, nodes_1.structTypeNode)([discriminatorArgument, ...struct.fields])),
            });
        },
        visitInstruction(node) {
            var _a;
            const shouldAddDiscriminator = (program === null || program === void 0 ? void 0 : program.origin) === 'anchor';
            if (!shouldAddDiscriminator)
                return node;
            const discriminatorArgument = (0, nodes_1.instructionArgumentNode)({
                name: 'discriminator',
                type: (0, nodes_1.arrayTypeNode)((0, nodes_1.numberTypeNode)('u8'), (0, nodes_1.fixedCountNode)(8)),
                defaultValue: (0, shared_1.getAnchorInstructionDiscriminator)(node.idlName),
                defaultValueStrategy: 'omitted',
            });
            return (0, nodes_1.instructionNode)({
                ...node,
                discriminators: [
                    (0, nodes_1.fieldDiscriminatorNode)('discriminator'),
                    ...((_a = node.discriminators) !== null && _a !== void 0 ? _a : []),
                ],
                arguments: [discriminatorArgument, ...node.arguments],
            });
        },
    }));
}
exports.setAnchorDiscriminatorsVisitor = setAnchorDiscriminatorsVisitor;
//# sourceMappingURL=setAnchorDiscriminatorsVisitor.js.map