"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setStructDefaultValuesVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
function setStructDefaultValuesVisitor(map) {
    return (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)(Object.entries(map).flatMap(([stack, defaultValues]) => {
        const mainCasedDefaultValues = Object.fromEntries(Object.entries(defaultValues).map(([key, value]) => [
            (0, shared_1.mainCase)(key),
            value,
        ]));
        return [
            {
                select: `${stack}.[structTypeNode]`,
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'structTypeNode');
                    const fields = node.fields.map((field) => {
                        const defaultValue = mainCasedDefaultValues[field.name];
                        if (defaultValue === undefined)
                            return field;
                        if (defaultValue === null) {
                            return (0, nodes_1.structFieldTypeNode)({
                                ...field,
                                defaultValue: undefined,
                                defaultValueStrategy: undefined,
                            });
                        }
                        return (0, nodes_1.structFieldTypeNode)({
                            ...field,
                            defaultValue: 'kind' in defaultValue ? defaultValue : defaultValue.value,
                            defaultValueStrategy: 'kind' in defaultValue ? undefined : defaultValue.strategy,
                        });
                    });
                    return (0, nodes_1.structTypeNode)(fields);
                },
            },
            {
                select: ['[instructionNode]', stack],
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'instructionNode');
                    const transformArguments = (arg) => {
                        const defaultValue = mainCasedDefaultValues[arg.name];
                        if (defaultValue === undefined)
                            return arg;
                        if (defaultValue === null) {
                            return (0, nodes_1.instructionArgumentNode)({
                                ...arg,
                                defaultValue: undefined,
                                defaultValueStrategy: undefined,
                            });
                        }
                        return (0, nodes_1.instructionArgumentNode)({
                            ...arg,
                            defaultValue: 'kind' in defaultValue ? defaultValue : defaultValue.value,
                            defaultValueStrategy: 'kind' in defaultValue ? undefined : defaultValue.strategy,
                        });
                    };
                    return (0, nodes_1.instructionNode)({
                        ...node,
                        arguments: node.arguments.map(transformArguments),
                        extraArguments: node.extraArguments
                            ? node.extraArguments.map(transformArguments)
                            : undefined,
                    });
                },
            },
        ];
    }));
}
exports.setStructDefaultValuesVisitor = setStructDefaultValuesVisitor;
//# sourceMappingURL=setStructDefaultValuesVisitor.js.map