"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.topDownTransformerVisitor = void 0;
const shared_1 = require("../shared");
const identityVisitor_1 = require("./identityVisitor");
const interceptVisitor_1 = require("./interceptVisitor");
const recordNodeStackVisitor_1 = require("./recordNodeStackVisitor");
function topDownTransformerVisitor(transformers, nodeKeys) {
    const transformerFunctions = transformers.map((transformer) => {
        if (typeof transformer === 'function')
            return transformer;
        return (node, stack) => (0, shared_1.getConjunctiveNodeSelectorFunction)(transformer.select)(node, stack)
            ? transformer.transform(node, stack)
            : node;
    });
    const stack = new shared_1.NodeStack();
    return (0, shared_1.pipe)((0, identityVisitor_1.identityVisitor)(nodeKeys), (v) => (0, recordNodeStackVisitor_1.recordNodeStackVisitor)(v, stack), (v) => (0, interceptVisitor_1.interceptVisitor)(v, (node, next) => {
        const appliedNode = transformerFunctions.reduce((acc, transformer) => acc === null ? null : transformer(acc, stack.clone()), node);
        if (appliedNode === null)
            return null;
        return next(appliedNode);
    }));
}
exports.topDownTransformerVisitor = topDownTransformerVisitor;
//# sourceMappingURL=topDownTransformerVisitor.js.map