"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformU8ArraysToBytesVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const extendVisitor_1 = require("./extendVisitor");
const nonNullableIdentityVisitor_1 = require("./nonNullableIdentityVisitor");
const visitor_1 = require("./visitor");
function transformU8ArraysToBytesVisitor(sizes = [32, 64]) {
    const hasRequiredSize = (count) => {
        if (!(0, nodes_1.isNode)(count, 'fixedCountNode'))
            return false;
        return sizes === '*' || sizes.includes(count.value);
    };
    return (0, shared_1.pipe)((0, nonNullableIdentityVisitor_1.nonNullableIdentityVisitor)(), (v) => (0, extendVisitor_1.extendVisitor)(v, {
        visitArrayType(node, { self }) {
            const child = (0, visitor_1.visit)(node.item, self);
            (0, nodes_1.assertIsNode)(child, nodes_1.TYPE_NODES);
            if ((0, nodes_1.isNode)(child, 'numberTypeNode') &&
                child.format === 'u8' &&
                (0, nodes_1.isNode)(node.count, 'fixedCountNode') &&
                hasRequiredSize(node.count)) {
                return (0, nodes_1.fixedSizeTypeNode)((0, nodes_1.bytesTypeNode)(), node.count.value);
            }
            return (0, nodes_1.arrayTypeNode)(child, node.count);
        },
    }));
}
exports.transformU8ArraysToBytesVisitor = transformU8ArraysToBytesVisitor;
//# sourceMappingURL=transformU8ArraysToBytesVisitor.js.map