"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateAccountsVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
function updateAccountsVisitor(map) {
    return (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)(Object.entries(map).flatMap(([selector, updates]) => {
        const newName = typeof updates === 'object' && 'name' in updates && updates.name
            ? (0, shared_1.mainCase)(updates.name)
            : undefined;
        const pdasToUpsert = [];
        const transformers = [
            {
                select: ['[accountNode]', selector],
                transform: (node, stack) => {
                    (0, nodes_1.assertIsNode)(node, 'accountNode');
                    if ('delete' in updates)
                        return null;
                    const { seeds, pda, ...assignableUpdates } = updates;
                    let newPda = node.pda;
                    if (pda && !pda.importFrom && seeds !== undefined) {
                        newPda = pda;
                        pdasToUpsert.push({
                            program: stack.getProgram().name,
                            pda: (0, nodes_1.pdaNode)(pda.name, seeds),
                        });
                    }
                    else if (pda) {
                        newPda = pda;
                    }
                    else if (seeds !== undefined && node.pda) {
                        pdasToUpsert.push({
                            program: stack.getProgram().name,
                            pda: (0, nodes_1.pdaNode)(node.pda.name, seeds),
                        });
                    }
                    else if (seeds !== undefined) {
                        newPda = (0, nodes_1.pdaLinkNode)(newName !== null && newName !== void 0 ? newName : node.name);
                        pdasToUpsert.push({
                            program: stack.getProgram().name,
                            pda: (0, nodes_1.pdaNode)(newName !== null && newName !== void 0 ? newName : node.name, seeds),
                        });
                    }
                    return (0, nodes_1.accountNode)({
                        ...node,
                        ...assignableUpdates,
                        data: (0, nodes_1.transformNestedTypeNode)(node.data, (struct) => { var _a; return (0, shared_1.renameStructNode)(struct, (_a = updates.data) !== null && _a !== void 0 ? _a : {}); }),
                        pda: newPda,
                    });
                },
            },
            {
                select: `[programNode]`,
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'programNode');
                    const pdasToUpsertForProgram = pdasToUpsert
                        .filter((p) => p.program === node.name)
                        .map((p) => p.pda);
                    if (pdasToUpsertForProgram.length === 0)
                        return node;
                    const existingPdaNames = new Set(node.pdas.map((pda) => pda.name));
                    const pdasToCreate = pdasToUpsertForProgram.filter((p) => !existingPdaNames.has(p.name));
                    const pdasToUpdate = new Map(pdasToUpsertForProgram
                        .filter((p) => existingPdaNames.has(p.name))
                        .map((p) => [p.name, p]));
                    const newPdas = [
                        ...node.pdas.map((p) => { var _a; return (_a = pdasToUpdate.get(p.name)) !== null && _a !== void 0 ? _a : p; }),
                        ...pdasToCreate,
                    ];
                    return (0, nodes_1.programNode)({ ...node, pdas: newPdas });
                },
            },
        ];
        if (newName) {
            transformers.push({
                select: ['[accountLinkNode]', selector],
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'accountLinkNode');
                    if (node.importFrom)
                        return node;
                    return (0, nodes_1.accountLinkNode)(newName);
                },
            }, {
                select: ['[pdaNode]', selector],
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'pdaNode');
                    return (0, nodes_1.pdaNode)(newName, node.seeds);
                },
            }, {
                select: ['[pdaLinkNode]', selector],
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'pdaLinkNode');
                    if (node.importFrom)
                        return node;
                    return (0, nodes_1.pdaLinkNode)(newName);
                },
            });
        }
        return transformers;
    }));
}
exports.updateAccountsVisitor = updateAccountsVisitor;
//# sourceMappingURL=updateAccountsVisitor.js.map