"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateDefinedTypesVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
function updateDefinedTypesVisitor(map) {
    return (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)(Object.entries(map).flatMap(([selector, updates]) => {
        const newName = typeof updates === 'object' && 'name' in updates && updates.name
            ? (0, shared_1.mainCase)(updates.name)
            : undefined;
        const transformers = [
            {
                select: ['[definedTypeNode]', selector],
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'definedTypeNode');
                    if ('delete' in updates) {
                        return null;
                    }
                    const { data: dataUpdates, ...otherUpdates } = updates;
                    let newType = node.type;
                    if ((0, nodes_1.isNode)(node.type, 'structTypeNode')) {
                        newType = (0, shared_1.renameStructNode)(node.type, dataUpdates !== null && dataUpdates !== void 0 ? dataUpdates : {});
                    }
                    else if ((0, nodes_1.isNode)(node.type, 'enumTypeNode')) {
                        newType = (0, shared_1.renameEnumNode)(node.type, dataUpdates !== null && dataUpdates !== void 0 ? dataUpdates : {});
                    }
                    return (0, nodes_1.definedTypeNode)({
                        ...node,
                        ...otherUpdates,
                        name: newName !== null && newName !== void 0 ? newName : node.name,
                        type: newType,
                    });
                },
            },
        ];
        if (newName) {
            transformers.push({
                select: ['[definedTypeLinkNode]', selector],
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'definedTypeLinkNode');
                    if (node.importFrom)
                        return node;
                    return (0, nodes_1.definedTypeLinkNode)(newName);
                },
            });
        }
        return transformers;
    }));
}
exports.updateDefinedTypesVisitor = updateDefinedTypesVisitor;
//# sourceMappingURL=updateDefinedTypesVisitor.js.map