import type { AccountNode } from './AccountNode';
import type { DefinedTypeNode } from './DefinedTypeNode';
import type { ErrorNode } from './ErrorNode';
import type { InstructionAccountNode } from './InstructionAccountNode';
import type { InstructionArgumentNode } from './InstructionArgumentNode';
import type { InstructionByteDeltaNode } from './InstructionByteDeltaNode';
import type { InstructionNode } from './InstructionNode';
import type { InstructionRemainingAccountsNode } from './InstructionRemainingAccountsNode';
import type { PdaNode } from './PdaNode';
import type { ProgramNode } from './ProgramNode';
import type { RootNode } from './RootNode';
import { RegisteredContextualValueNode } from './contextualValueNodes/ContextualValueNode';
import { RegisteredCountNode } from './countNodes/CountNode';
import { RegisteredDiscriminatorNode } from './discriminatorNodes/DiscriminatorNode';
import { RegisteredLinkNode } from './linkNodes/LinkNode';
import { RegisteredPdaSeedNode } from './pdaSeedNodes/PdaSeedNode';
import { RegisteredTypeNode } from './typeNodes/TypeNode';
import { RegisteredValueNode } from './valueNodes/ValueNode';
export type Node = RegisteredContextualValueNode | RegisteredCountNode | RegisteredDiscriminatorNode | RegisteredLinkNode | RegisteredPdaSeedNode | RegisteredTypeNode | RegisteredValueNode | RootNode | ProgramNode | PdaNode | AccountNode | InstructionAccountNode | InstructionArgumentNode | InstructionByteDeltaNode | InstructionNode | InstructionRemainingAccountsNode | ErrorNode | DefinedTypeNode;
export declare const REGISTERED_NODE_KINDS: ("arrayValueNode" | "bytesValueNode" | "booleanValueNode" | "constantValueNode" | "enumValueNode" | "mapValueNode" | "noneValueNode" | "numberValueNode" | "setValueNode" | "someValueNode" | "structValueNode" | "tupleValueNode" | "publicKeyValueNode" | "stringValueNode" | "accountBumpValueNode" | "accountValueNode" | "argumentValueNode" | "conditionalValueNode" | "identityValueNode" | "payerValueNode" | "pdaValueNode" | "programIdValueNode" | "resolverValueNode" | "pdaSeedValueNode" | "fixedCountNode" | "remainderCountNode" | "prefixedCountNode" | "constantDiscriminatorNode" | "fieldDiscriminatorNode" | "sizeDiscriminatorNode" | "programLinkNode" | "pdaLinkNode" | "accountLinkNode" | "definedTypeLinkNode" | "constantPdaSeedNode" | "variablePdaSeedNode" | "amountTypeNode" | "arrayTypeNode" | "booleanTypeNode" | "bytesTypeNode" | "dateTimeTypeNode" | "enumTypeNode" | "fixedSizeTypeNode" | "hiddenPrefixTypeNode" | "hiddenSuffixTypeNode" | "mapTypeNode" | "numberTypeNode" | "optionTypeNode" | "postOffsetTypeNode" | "preOffsetTypeNode" | "publicKeyTypeNode" | "sentinelTypeNode" | "setTypeNode" | "sizePrefixTypeNode" | "solAmountTypeNode" | "stringTypeNode" | "structTypeNode" | "tupleTypeNode" | "zeroableOptionTypeNode" | "structFieldTypeNode" | "enumEmptyVariantTypeNode" | "enumStructVariantTypeNode" | "enumTupleVariantTypeNode" | "mapEntryValueNode" | "structFieldValueNode" | "rootNode" | "programNode" | "pdaNode" | "accountNode" | "instructionAccountNode" | "instructionArgumentNode" | "instructionByteDeltaNode" | "instructionNode" | "instructionRemainingAccountsNode" | "errorNode" | "definedTypeNode")[];
export type NodeKind = Node['kind'];
export type GetNodeFromKind<TKind extends NodeKind> = Extract<Node, {
    kind: TKind;
}>;
export declare function isNode<TKind extends NodeKind>(node: Node | null | undefined, kind: TKind | TKind[]): node is GetNodeFromKind<TKind>;
export declare function assertIsNode<TKind extends NodeKind>(node: Node | null | undefined, kind: TKind | TKind[]): asserts node is GetNodeFromKind<TKind>;
export declare function isNodeFilter<TKind extends NodeKind>(kind: TKind | TKind[]): (node: Node | null | undefined) => node is GetNodeFromKind<TKind>;
export declare function assertIsNodeFilter<TKind extends NodeKind>(kind: TKind | TKind[]): (node: Node | null | undefined) => node is GetNodeFromKind<TKind>;
export declare function removeNullAndAssertIsNodeFilter<TKind extends NodeKind>(kind: TKind | TKind[]): (node: Node | null | undefined) => node is GetNodeFromKind<TKind>;
